import calendar
import time
from pathlib import Path

import lightgbm as lgb
from pandas import read_csv
from sklearn.metrics import r2_score
from sklearn.model_selection import train_test_split

from real_estate_model.config import Config, fetch_config_from_yaml


def calc_metrics(actual, pred, suffix):
    # rmse = sqrt(mean_squared_error(actual, pred))
    r2 = r2_score(actual, pred)
    # print(f'{suffix} rmse: {rmse}')
    print(f"{suffix} R2: {r2}")


def train() -> None:
    # getting train params
    print()
    config = Config(**fetch_config_from_yaml().data)

    # read training data
    PACKAGE_ROOT = Path(__file__).resolve().parent
    if Path(config.dataset).is_file():
        data = read_csv(config.dataset)
    else:
        print(f'ERROR! Specify dataset path - file "{config.dataset}" does not exists!')
        return

    # process data
    price = data[data["price"] <= 0].index
    data.drop(price, inplace=True)

    rooms = data[data["rooms"] <= 0].index
    data.drop(rooms, inplace=True)

    price_high = data[data["price"] >= 20.000000e06].index
    data.drop(price_high, inplace=True)

    # drop variables to drop
    labels_to_drop = config.variables_to_drop
    labels_to_drop.append("price")
    X = data.drop(labels=labels_to_drop, axis=1)
    Y = data["price"]

    # divide train and test
    X_train, X_test, y_train, y_test = train_test_split(
        X, Y, test_size=config.train_size, random_state=config.random_state
    )

    # fit model
    lgb_train = lgb.Dataset(X_train, y_train)
    lgb_eval = lgb.Dataset(X_test, y_test)
    params = {"metric": config.metric}
    model = lgb.train(
        params,
        lgb_train,
        valid_sets=lgb_eval,
        num_boost_round=config.num_boost_round,
        early_stopping_rounds=config.early_stopping_rounds,
        verbose_eval=config.verbose_eval,
    )
    print()

    # make predictions for train set
    train_prediction = model.predict(X_train)
    calc_metrics(y_train, train_prediction, "train")

    # make predictions for test set
    test_prediction = model.predict(X_test)
    calc_metrics(y_test, test_prediction, "test")
    print()

    # persist trained model
    timestamp = calendar.timegm(time.gmtime())
    model_filename = config.trained_model_path + "lightgbm_" + str(timestamp) + ".txt"
    TRAINED_FILE_PATH = PACKAGE_ROOT / model_filename
    model.save_model(TRAINED_FILE_PATH)
    print(f"trained model saved to {model_filename}")


if __name__ == "__main__":
    train()



from pathlib import Path
from typing import List

from pydantic import BaseModel
from strictyaml import YAML, load


class Config(BaseModel):
    dataset: str
    trained_model_path: str

    features: List[str]
    variables_to_drop: List[str]

    random_state: int
    train_size: float

    num_boost_round: int
    early_stopping_rounds: int
    verbose_eval: int
    metric: str

    predictors: List[str]
    predict_model: str


def fetch_config_from_yaml(cfg_path: Path = None) -> YAML:
    PACKAGE_ROOT = Path(__file__).resolve().parent
    CONFIG_FILE_PATH = PACKAGE_ROOT / "config.yml"

    if not cfg_path:
        cfg_path = CONFIG_FILE_PATH

    if cfg_path:
        with open(cfg_path, "r") as conf_file:
            parsed_config = load(conf_file.read())
            return parsed_config
    raise OSError(f"Did not find config file at path: {cfg_path}")



from pathlib import Path

import lightgbm as lgb
import pandas as pd

from real_estate_model.config import Config, fetch_config_from_yaml

config = Config(**fetch_config_from_yaml().data)
PACKAGE_ROOT = Path(__file__).resolve().parent
MODEL_FILE_PATH = PACKAGE_ROOT / config.predict_model
model = lgb.Booster(model_file=MODEL_FILE_PATH)


# geo_lat float
# geo_lon float
# Region uint categoric
#   Region of Russia. There are 85 subjects in the country in total.
# level uint
# levels uint
# rooms uint
# area float
# object_type uint categoric
#   1 = secondary build
#   11 = new build
def predict(predictors_values):
    data = pd.DataFrame([predictors_values], columns=config.predictors)
    case = pd.DataFrame(data=data, index=[0])
    prediction = model.predict(case)
    return prediction[0]


if __name__ == "__main__":
    price = predict([80.805808, 30.376141, 2661, 8, 10, 3, 82.6, 1])
    print(f"{price:.2f} Rub")



from pathlib import Path
from setuptools import setup, find_packages

def readme():
  with open('README.md', 'r') as f:
    return f.read()

ROOT_DIR = Path(__file__).resolve().parent
REQUIREMENTS_DIR = ROOT_DIR / "requirements"
def list_reqs(fname="requirements.txt"):
    with open(REQUIREMENTS_DIR / fname) as fd:
        return fd.read().splitlines()

setup(
  name='RealEstate_package',
  version='1.4.0',
  license="BSD-3",
  author='Клейменов А., Толстенко Л.',
  author_email='notmy@gmail.com',
  description='МЛ-модель, предсказывающая стоимость недвижимости по её параметрам.',
  long_description=readme(),
  long_description_content_type='text/markdown',
  url='https://github.com/Lada-Rom/RealEstate_package',
  packages=find_packages(),
  include_package_data=True,
  install_requires=list_reqs(),
  extras_require={},
  classifiers=[
    'Programming Language :: Python :: 3.8',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent'
  ],
  keywords='lightgbm python real_estate price',
  project_urls={
    'Documentation': 'https://github.com/Lada-Rom/RealEstate_package'
  },
  python_requires='>=3.7'
)
